﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Split
{
    class SplitFile
    {
        private string _fileName;
        FileStream _stream;
        BinaryWriter _writer;

        /// <summary>
        /// Процесс разбиения файла
        /// </summary>
        /// <param name="fileName">Имя разбиваемого файла</param>
        /// <param name="partSize">Размер одной части в байтах</param>
        public void Do(string fileName, long partSize)
        {
            _fileName = fileName;
            byte b;
            using (var _reader = new BinaryReader(new FileStream(_fileName, FileMode.Open, FileAccess.Read), Encoding.Default))
            {
                long fileSize = _reader.BaseStream.Length;
                if (partSize >= fileSize)
                {
                    Console.WriteLine("Nothing to do. File size is less then {0} bytes", partSize);
                    return;
                }
                    

                double partCount = Math.Floor( (double)(fileSize/partSize) ) ;
                int currentPart = 0;
                long position = 0;
                long mod = 0;
                try
                {
                    // читаем весь файл по байту
                    while (true)
                    {
                        b = _reader.ReadByte();


                        Math.DivRem(position, partSize, out mod);
                        if (mod == 0 && position > 0)
                            currentPart++;

                        Write(currentPart, b);

                        position++;
                    }
                }
                catch (EndOfStreamException e)
                {
                    Console.WriteLine("File was splitted to {0} parts", partCount + 1);
                }

            }
        }

        private int _prevPart = -1;
        public void Write(int part, byte b)
        {
            if (_prevPart != part)
            {
                _prevPart = part;
                if (_writer != null)
                    _writer.Dispose();
                _writer = new BinaryWriter(new FileStream(_fileName + part, FileMode.Create, FileAccess.Write), Encoding.Default);
            }

            _writer.Write(b);
        }


    }
}
